/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/broker.h
 *  
 *  Description:
 *    <--->
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifndef broker_h_included
#define broker_h_included
#include "depot.h"
#include "operators.h"

#include <string>
#include <map>

namespace Binc {

  //------------------------------------------------------------------
  class Command;

  //------------------------------------------------------------------
  class Broker;
  
  //------------------------------------------------------------------
  class BrokerFactory {

    //--
    std::map<int, Broker *> brokers;

    //--
    BrokerFactory(void);

  public:

    //--    
    Broker &getBroker(int state);

    //--
    void assign(Operator *o);

    //--
    Operator * getOperator(int state, const std::string &name) const;

    void addCapability(const std::string &c);

    //--
    static BrokerFactory &getInstance(void);

    //--
    ~BrokerFactory(void);
  };

  //------------------------------------------------------------------
  class Broker {

    //--
    std::map<std::string, Operator *> operators;

    //--
    std::map<std::string, bool> deletables;

  public:

    //--
    Operator * get(const std::string &name) const;

    //--
    void assign(Operator *o, bool deletable = false);

    //--
    bool parse(Command &) const;

    //--
    bool process(Depot &, Command &)
      const throw (OperatorException &);

    //--
    Broker(void);
    ~Broker(void);
  };
}

#endif
