/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/depot.h
 *  
 *  Description:
 *    <--->
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifndef depot_h_included
#define depot_h_included
#include <map>
#include <string>
#include <vector>

#include <dirent.h>

#include "exceptions.h"

namespace Binc {

  //------------------------------------------------------------------
  class Mailbox;

  //------------------------------------------------------------------
  class Depot;

  //------------------------------------------------------------------
  class Status;

  //------------------------------------------------------------------
  class DepotFactory {

    //--
    std::vector<Depot *> depots;

    //--
    DepotFactory(void);

  public:

    //--
    void assign(Depot *);

    //--
    Depot *get(const std::string &name) const;

    //--
    static DepotFactory &getInstance(void);

    //--
    ~DepotFactory(void);
  };

  //------------------------------------------------------------------
  class Depot {
  public:
    //--
    class iterator {

      //--
      DIR *dirp;
      struct dirent *direntp;

    public:      

      //--
      std::string operator * (void) const;
      void operator ++ (void);
      bool operator != (iterator) const;
      bool operator == (iterator) const;

      //--
      iterator(void);
      iterator(DIR *, struct dirent *);

      friend class Depot;
    };

  private:

    //--
    std::vector<Mailbox *> backends;

    //--
    Mailbox *defaultmailbox;
    Mailbox *selectedmailbox;

    //--
    iterator enditerator;

  protected:

    //--
    std::string name;

    //--
    char delimiter;


    //--
    mutable std::map<std::string, Status> mailboxstatuses;

  public:

    //--
    iterator begin(const std::string &) const;

    //--
    const iterator &end(void) const;

    //--
    void setDelimiter(char);

    //--
    void assign(Mailbox *);

    //--
    void setSelected(Mailbox *) throw (Binc::CriticalException &);

    //--
    void setDefaultType(const std::string &n) 
      throw (Binc::CriticalException &);

    //--
    Mailbox *getDefault(void) const;

    //--
    Mailbox *get(const std::string &path) const;

    //--
    Mailbox *getSelected(void) const;

    //--
    Status &getStatus(const std::string &s_in) const;

    //--
    const char getDelimiter(void) const;

    //--
    const std::string &getName(void) const;

    //--
    virtual std::string mailboxToFilename(const std::string &m) const = 0;

    //--
    virtual std::string filenameToMailbox(const std::string &m) const = 0;

    //--
    virtual int createMailbox(const std::string &m) const
      throw (CriticalException &, StorageException &);

    //--
    virtual int deleteMailbox(const std::string &m) const
      throw (CriticalException &, StorageException &);

    //--
    virtual int renameMailbox(const std::string &m,
			      const std::string &n) const
      throw (StorageException &);

    //--
    Depot(void);
    Depot(const std::string &name);
    virtual ~Depot(void);
  };

  //------------------------------------------------------------------
  class MaildirPPDepot : public Depot {

  public:

    //--
    std::string mailboxToFilename(const std::string &m) const;

    //--
    std::string filenameToMailbox(const std::string &m) const;

    //--
    MaildirPPDepot();
    ~MaildirPPDepot();
  };

  //------------------------------------------------------------------
  class IMAPdirDepot : public Depot {

  public:

    //--
    std::string mailboxToFilename(const std::string &m) const;

    //--
    std::string filenameToMailbox(const std::string &m) const;

    //--
    IMAPdirDepot();
    ~IMAPdirDepot();
  };

}

#endif
