/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/exceptions.cc
 *  
 *  Description:
 *    Implementation of common exceptions
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#include <string>

#include "exceptions.h"

using namespace std;

//------------------------------------------------------------------------
Binc::BincException::BincException(const string &s_in)
{
  reason = s_in;
}

//------------------------------------------------------------------------
Binc::BincException::~BincException(void) throw ()
{
}

//------------------------------------------------------------------------
void Binc::BincException::addReason(const string &s_in)
{
  reason = s_in + string(", ") + reason;
}

//------------------------------------------------------------------------
const string &Binc::BincException::getReason(void) const
{
  return reason;
}

//------------------------------------------------------------------------
Binc::CriticalException::CriticalException(const string &s_in)
  : BincException(s_in)
{
}

//------------------------------------------------------------------------
Binc::CriticalException::~CriticalException(void) throw ()
{
}

//------------------------------------------------------------------------
Binc::InputException::InputException(const string &s_in) 
  : BincException(s_in)
{
}

//------------------------------------------------------------------------
Binc::InputException::~InputException(void) throw ()
{
}

//------------------------------------------------------------------------
Binc::MessageException::MessageException(const string &s_in)
  : BincException(s_in)
{
}

//------------------------------------------------------------------------
Binc::MessageException::~MessageException(void) throw ()
{
}


//------------------------------------------------------------------------
Binc::OperatorException::OperatorException(int c_in, const string &s_in)
  : BincException(s_in)
{
  code = c_in;
}

//------------------------------------------------------------------------
Binc::OperatorException::~OperatorException(void) throw ()
{
}

//------------------------------------------------------------------------
int Binc::OperatorException::getCode(void) const
{
  return code;
}

//------------------------------------------------------------------------
Binc::StorageException::StorageException(const string &s_in)
  : BincException(s_in)
{
}

//------------------------------------------------------------------------
Binc::StorageException::~StorageException(void) throw ()
{
}

//------------------------------------------------------------------------
Binc::SyntaxException::SyntaxException(const string &s_in)
  : BincException(s_in)
{
}

//------------------------------------------------------------------------
Binc::SyntaxException::~SyntaxException(void) throw ()
{
}

//------------------------------------------------------------------------
Binc::ModuleException::ModuleException(const std::string &m, const std::string &dlerror) :
  BincException(m + ": " + dlerror)
{
}

//------------------------------------------------------------------------
Binc::ModuleException::~ModuleException(void) throw ()
{
}

//------------------------------------------------------------------------
Binc::TimeoutException::TimeoutException(const std::string &s_in) : BincException(s_in)
{
}

//------------------------------------------------------------------------
Binc::TimeoutException::~TimeoutException(void) throw ()
{
}
