/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/exceptions.h
 *  
 *  Description:
 *    Declaration of common exceptions
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifndef exceptions_h_included
#define exceptions_h_included
#include <exception>
#include <string>
#include <iostream>

namespace Binc {

  //------------------------------------------------------------------------
  class BincException : public std::exception {

    //--
    std::string reason;

  public:

    //--
    void addReason(const std::string &reason_in);
    const std::string &getReason(void) const;

    //--
    BincException(const std::string &reason_in);
    virtual ~BincException(void) throw ();
  };

  //------------------------------------------------------------------------
  class CriticalException : public BincException {
  public:

    //--
    CriticalException(const std::string &reason_in);
    ~CriticalException(void) throw ();
  };

  //----------------------------------------------------------------------
  class InputException : public BincException {
  public:

    //--
    InputException(const std::string &reason_in);
    ~InputException(void) throw ();
  };

  //------------------------------------------------------------------------
  class MessageException : public BincException {
  public:

    //--
    MessageException(const std::string &s_in);
    ~MessageException() throw ();
  };

  //------------------------------------------------------------------------
  class OperatorException : public BincException {

    //--
    int code;

  public:
    //--
    int getCode(void) const;

    //--
    OperatorException(int code, const std::string &reason_in);
    ~OperatorException(void) throw ();
  };

  //------------------------------------------------------------------------
  class StorageException : public BincException {
  public:

    //--
    StorageException(const std::string &reason);
    ~StorageException() throw ();
  };

  //------------------------------------------------------------------------
  class SyntaxException : public BincException {
  public:

    //--
    SyntaxException(const std::string &reason_in);
    ~SyntaxException(void) throw ();
  };
  
  class ModuleException : public BincException {
  public:
    ModuleException(const std::string &m, const std::string &dlerror);
    ~ModuleException(void) throw ();
  };

  class TimeoutException : public BincException {
  public:
    TimeoutException(const std::string &s_in);
    ~TimeoutException(void) throw ();
  };

}


#endif
