/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/mailbox/maildir-close.cc
 *  
 *  Description:
 *    Implementation of the Maildir class.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "../../parsers/storage/storage.h"
#include "../../io/io.h"
#include "maildir.h"

#include <fcntl.h>
#include <unistd.h>

using namespace ::std;
using namespace Binc;

//------------------------------------------------------------------------
void Binc::Maildir::imapClose(void)
{
  if (!selected)
    return;

  const string uidvalfilename = path + "/bincimap-uidvalidity";
  
  writeCache();

  try {
    Storage uidvalfile;
    uidvalfile.setSection("depot");
    uidvalfile.insert("_uidvalidity", toString(uidvalidity));
    uidvalfile.insert("_uidnext", toString(uidnext));
    uidvalfile.insert("_version", UIDVALFILEVERSION);
    uidvalfile.save(uidvalfilename);
  } catch (StorageException &e) {
    IO &logger = IOFactory::getInstance().get(2);

    logger << "warning in writeCache, while saving to " << uidvalfilename 
	   << ": " << e.getReason() << endl;
    throw e;
  } 

  Mailbox::imapClose();
   
  oldrecent = 0;
  oldexists = 0;
  firstscan = true;
  cacheRead = false;
  uidvalidity = -1;
  uidnext = 1;
  selected = false;
}
