/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/mailbox/maildir.h
 *  
 *  Description:
 *    Declaration of the Maildir class.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef maildir_h_included
#define maildir_h_included
#include <string>
#include <vector>
#include <map>

#include "../../exceptions.h"
#include "../mailbox.h"
#include "../message.h"

namespace Binc {
  static const std::string CACHEFILEVERSION = "1.0.3";
  static const std::string UIDVALFILEVERSION = "1.0.3";

  //------------------------------------------------------------------------
  class Maildir : public Mailbox {

    //--
    mutable std::map<std::string, std::string> fixedtofile;
    mutable int uidvalidity;
    mutable int uidnext;
    mutable bool firstscan;
    mutable bool cacheRead;

    //--
    bool selected;

    //--
    std::string path;

    //--
    bool readCache(void) const;

    //--
    void writeCache(void) const;

    //--
    void scanFileNames(void) const;

  public:

    //--
    inline const std::string &getUidValidity(void) const
    {
      if (uidvalidity == -1)
	scan();
      static std::string uidv;
      return (uidv = toString(uidvalidity));
    }

    //--
    inline int getUidNext(void) const
    {
      if (uidnext == -1)
	scan();
      
      return uidnext;
    }

    //--
    PendingUpdates getUpdates(bool doscan, int type);

    //--
    const std::string &getPath(void) const;

    //--
    int open(const std::string &id);

    //--
    FILE *fopen(const std::string &id) const;

    //--
    void setPath(const std::string &path_in);

    //--
    const std::string getName(void) const;

    //--
    void bumpUidValidity(const std::string &) const;

    //--
    bool isMailbox(const std::string &) const;

    //--
    bool isMarked(const std::string &) const;

    //--
    int getStatusID(const std::string &) const;

    //--
    bool getStatus(const std::string &, Status &) const
      throw (StorageException &);

    //--
    void scan(void) const;

    //--
    void updateFlags(void);

    //--
    void imapSelect(const std::string &s_in);

    //--
    void imapClose(void);

    //--
    void imapExpunge(void);

    //--
    void imapCreate(const std::string &s, mode_t mode,
		    uid_t owner = 0, gid_t group = 0, bool root = false);

    //--
    void imapDelete(const std::string &s);

    //--
    Maildir(void);
    virtual ~Maildir(void);
  };
}

#endif
