/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/operator-subscribe.cc
 *  
 *  Description:
 *    Implementation of the SUBSCRIBE command.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <iostream>

#include "parsers/storage/storage.h"
#include "io/io.h"
#include "util/convert.h"

#include <sys/stat.h>

#include "parsers/imap/recursivedescent/recursivedescent.h"

#include "session.h"
#include "depot.h"
#include "operators.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
SubscribeOperator::SubscribeOperator(void)
{
}

//----------------------------------------------------------------------
SubscribeOperator::~SubscribeOperator(void)
{
}

//----------------------------------------------------------------------
const string SubscribeOperator::getName(void) const
{
  return "SUBSCRIBE";
}

//----------------------------------------------------------------------
int SubscribeOperator::getState(void) const
{
  return Session::AUTHENTICATED | Session::SELECTED;
}

//------------------------------------------------------------------------
void SubscribeOperator::process(Depot &depot,
				Command &command) const
{
  Session &session = Session::getInstance();
  IO &logger = IOFactory::getInstance().get(2);

  const string &srcmailbox = command.getMailbox();
  const string &canonmailbox = toCanonMailbox(srcmailbox);

  session.subscribed.push_back(canonmailbox);
  
  Storage subs;
  subs.setSection("subscribed");
  int n = 0;
  for (vector<string>::const_iterator i = session.subscribed.begin();
       i != session.subscribed.end(); ++i) {
    subs.insert(toString(n), *i);
    ++n;
  }
  
  try {
    subs.save("bincimap-subscribed");
  } catch (StorageException &e) {
    logger << "Saving bincimap-subscribed failed: " << e.getReason() << endl;
  }
}

//----------------------------------------------------------------------
bool SubscribeOperator::parse(Command &c_in) const
{
  string mailbox;

  try {
    if (!expectThisString("SUBSCRIBE"))
      return false;

    if (!expectSPACE())
      throw SyntaxException("expected SPACE");

    if (!expectMailbox(mailbox))
      throw SyntaxException("expected mailbox");    
    if (!expectCRLF())
      throw SyntaxException("expected CRLF");

  } catch (InputException &e) {
    e.addReason("in SubscribeOperator::parse");
    throw e;
  }

  c_in.setName("SUBSCRIBE");
  c_in.setMailbox(mailbox);

  return true;
}
