/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/parsers/imap/recursivedescent/recursivedescent.h
 *  
 *  Description:
 *    Declaration of a recursive descent IMAP command
 *    parser.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef expectcommand_h_inluded
#define expectcommand_h_inluded
#include <stack>
#include <string>

#include "../imapparser.h"

namespace Binc {

  extern std::stack<int> inputBuffer;
  extern int charnr;
    
  int readChar(void);
  void unReadChar(int c_in);
  void unReadChar(const std::string &s_in);

  bool expectTag(std::string &s_in);
  bool expectTagChar(int &c_in);
  bool expectSPACE(void);

  bool expectFlag(std::vector<std::string> &v_in);

  bool expectListMailbox(std::string &s_in);
  bool expectListWildcards(int &c_in);

  bool expectDateTime(std::string &s_in);
  bool expectTime(std::string &s_in);
  bool expectZone(std::string &s_in);

  bool expectMailbox(std::string &s_in);
  bool expectAstring(std::string &s_in);
  bool expectAtom(std::string &s_in);
  bool expectAtomChar(int &i_in);
  bool expectString(std::string &s_in);
    
  bool expectDate(std::string &s_in);

  bool expectNumber(int &i_in);
  bool expectDigit(int &i_in);
  bool expectDigitNZ(int &i_in);

  bool expectLiteral(std::string &s_in);
  bool expectQuoted(std::string &s_in);
  bool expectQuotedChar(int &c_in);

  bool expectSet(BincImapParserSet &s_in);
  bool expectSequenceNum(int &i_in);
  bool expectNZNumber(int &i_in);

  bool expectCRLF(void);
  bool expectCR(void);
  bool expectLF(void);

  bool expectThisString(const std::string &s_in);
}


#endif
