/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/parsers/mime/mime-printheader.cc
 *  
 *  Description:
 *    Implementation of main mime parser components
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mime.h"
#include "mime-utils.h"
#include "../../util/convert.h"
#include "../../io/io.h"
#include <string>
#include <vector>
#include <map>
#include <exception>
#include <iostream>

#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <errno.h>

using namespace ::std;

//------------------------------------------------------------------------
int Binc::MimePart::printHeader(FILE *fp,
				IO &output, vector<string> headers, bool includeheaders, 
				int startoffset, int length,
				bool doprint, bool dostore, string &store) const
{
  IO &com = IOFactory::getInstance().get(1);

  while (CRLFbuffer.size() != 0)
    CRLFbuffer.pop();
  CRLFlastchar = '\0';

  int c = '\0';
  fseek(fp, headerstartoffset, SEEK_SET);

  if (length == -1) length = headerlength;
  if (startoffset + length > headerlength)
    length = headerlength - startoffset + 1;

  int wrotebytes = 0;
  int processedbytes = 0;

  int crlfnr = 0;
  string name;
  string content;
  bool quit = false;

  bool breakheader = false;

  while (!quit) {

    // 1) read all characters to but not including ':'
    string name;
    if (crlfnr >= headerlength) {
      quit = true;
      break;
    }

    while (1) {
      if ((c = fgetcCRLF(fp)) == EOF)
	break;

      ++crlfnr;     
      if (c == ':') break;
      if (c == '\n') {
	for (int i = name.length() - 1; i >= 0; --i) {
	  fungetcCRLF(name[i]);
	  --crlfnr;
	}
	quit = true;
	breakheader = true;
	name = "";
	break;
      }

      if (c == EOF) {
	quit = true;
	break;
      }

      if (c != '\r')
	name += c;

      if (name.length() == 2 && name.substr(0, 2) == "\r\n") {
	name = "";
	quit = true;
	break;
      }
    }
    if (quit || c == EOF)
      break;

    // 2) check if we want to print this header
    string tmp = name;
    lowercase(tmp);
    bool match = false;
    if (headers.size() == 0) {
      match = true;
    } else {
      for (vector<string>::const_iterator i = headers.begin(); i != headers.end(); ++i) {
	string t2 = *i; lowercase(t2);

	if (t2 == tmp) {
	  match = true;
	  break;
	}
      }
      if (!includeheaders)
	if (match) match = false;
	else if (!match) match = true;
    }

    // 3) if we did not want to print this header, skip to the next header
    if (!match) {
      char cq[3];
      do {
	if (crlfnr >= headerlength) {
	  quit = true;
	  break;
	}

	if ((c = fgetcCRLF(fp)) == EOF)
	  break;

	crlfnr++;

	for (int i = 0; i < 2; ++i) cq[i] = cq[i + 1];
	cq[2] = c;
	if (strncmp(cq, "\r\n", 2) == 0) {
	  if (!isspace(c)) {
	    fungetcCRLF(c);
	    --crlfnr;
	    break;
	  }
	}
      } while (1);
      if (c == EOF) break;
    } else {
      // don't forget the colon
      name += ":";

      // 3.1) print what we have already
      for (string::const_iterator i = name.begin(); i != name.end(); ++i)
	if (processedbytes >= startoffset) {
	  if (doprint)
	    com << *i;
	  if (dostore)
	    store += *i;
	  ++wrotebytes;
	} else
	  ++processedbytes;
      
      // 3.2) print the rest of this header
      char cq[4];
      cq[2] = ':';
      do {
	if (crlfnr >= headerlength) {
	  quit = true;
	  break;
	}

	if ((c = fgetcCRLF(fp)) == EOF)
	  break;
	crlfnr++;

	for (int i = 0; i < 2; ++i) cq[i] = cq[i + 1];
	cq[2] = c;
	if (strncmp(cq, "\r\n", 2) == 0) {
	  if (!isspace(c)) {
	    fungetcCRLF(c);
	    --crlfnr;
	    break;
	  }
	}

	if (processedbytes >= startoffset) {
	  if (doprint)
	    com << (char)c;
	  if (dostore)
	    store += (char)c;
	  ++wrotebytes;
	} else
	  ++processedbytes;
      } while (1);
      if (c == EOF) break;
    }

    name = "";
  }

  if (breakheader || headers.size() != 0) {
    name = "\r\n";
    for (string::const_iterator i = name.begin(); i != name.end(); ++i)
      if (processedbytes >= startoffset) {
	if (doprint)
	  com << *i;
	if (dostore)
	  store += *i;
	++wrotebytes;
      } else
	++processedbytes;
  }

  return wrotebytes;
}
