/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/parsers/mime/mime-printstruct.cc
 *  
 *  Description:
 *    Implementation of main mime parser components
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mime.h"
#include "../../util/convert.h"
#include "../../io/io.h"
#include <string>
#include <vector>
#include <map>
#include <exception>
#include <iostream>

#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <errno.h>

using namespace ::std;

//------------------------------------------------------------------------
void Binc::MimePart::printStruct(string s) const
{
  IO &logger = IOFactory::getInstance().get(2);

  static int level = 0;

  HeaderItem hi;

  if (h.getFirstHeader("content-type", hi))
    logger << string(level, ' ') << "Content-Type: " << hi.getValue() << endl;
  if (h.getFirstHeader("subject", hi))
    logger << string(level, ' ') << "Subject: " << hi.getValue() << endl;
  logger << endl;

  if (isMultipart()) {
    if (members.size() != 0) {
      vector<MimePart>::const_iterator i = members.begin();
      level += 2;
      int part = 1;
      while (i != members.end()) {
	BincStream ss;

	ss << s;
	if (s != "")
	  ss << ".";
	ss << part;

	logger << string(level, ' ') << "Mime part: ";
	logger << ss.str() << endl;

	(*i).printStruct(ss.str());
	++i;
	part++;
      }
      level -= 2;
    }
  } else if (isMessageRFC822()) {
    if (members.size() != 0) {
      level += 2;
      members[0].printStruct(s);
      level -= 2;
    }
  } else {
    // Singlepart
    logger << string(level, ' ') << "Mime part: " << s;
    if (s != "")
      logger << ".";
    logger << "1" << endl;
  }
}
