/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/pendingupdates.cc
 *  
 *  Description:
 *    <--->
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#include <iostream>
#include <string>
#include <vector>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "pendingupdates.h"
#include "mailbox/mailbox.h"
#include "io/io.h"

using namespace ::std;
using namespace Binc;

//--------------------------------------------------------------------
void Binc::pendingUpdates(Mailbox *mailbox, int type, bool rescan)
{
  IO &com = IOFactory::getInstance().get(1);

  if (mailbox == NULL)
    throw CriticalException("pending updates on mailbox == NULL");

  PendingUpdates p = mailbox->getUpdates(rescan, type);

  if (type & PendingUpdates::EXPUNGE) {
    PendingUpdates::expunged_const_iterator i = p.beginExpunged();
    PendingUpdates::expunged_const_iterator e = p.endExpunged();

    while (i != e) {
      com << "* " << *i << " EXPUNGE" << endl;
      ++i;
    }
  }

  if ((type & PendingUpdates::EXISTS) && p.newExists())
    com << "* " << p.getExists() << " EXISTS" << endl;

  if ((type & PendingUpdates::RECENT) && p.newRecent())
    com << "* " << p.getRecent() << " RECENT" << endl;

  if (type & PendingUpdates::FLAGS) {
    PendingUpdates::flagupdates_const_iterator i = p.beginFlagUpdates();
    PendingUpdates::flagupdates_const_iterator e = p.endFlagUpdates();

    while (i != e) {
      int flags = i.second();

      vector<string> flagv;
      if (flags & Message::F_SEEN) flagv.push_back("\\Seen");
      if (flags & Message::F_ANSWERED) flagv.push_back("\\Answered");
      if (flags & Message::F_DELETED) flagv.push_back("\\Deleted");
      if (flags & Message::F_DRAFT) flagv.push_back("\\Draft");
      if (flags & Message::F_RECENT) flagv.push_back("\\Recent");
      if (flags & Message::F_FLAGGED) flagv.push_back("\\Flagged");

      com << "* " << i.first() << " FETCH (FLAGS (";  
      for (vector<string>::const_iterator k
	     = flagv.begin(); k != flagv.end(); ++k) {
	if (k != flagv.begin()) com << " ";
	com << *k;
      }
	
      com << "))" << endl;

      ++i;
    }
  }
}
