/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/util/convert.cc
 *  
 *  Description:
 *    Implementation of miscellaneous convertion functions.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "convert.h"
#include "../io/io.h"
#include <string>

#ifdef HAVE_SSTREAM
#include <sstream>
#else
#include <strstream>
#endif

#ifdef HAVE_OSTREAM
#include <ostream>
#include <streambuf>
#else
#include <ostream.h>
#include <streambuf.h>
#endif

using namespace ::std;
using namespace Binc;

//------------------------------------------------------------------------
class dummybuf : public std::streambuf {
};

//------------------------------------------------------------------------
BincStream::BincStream(void) : std::ostream(new dummybuf())
{
  terminated = false;
}

//------------------------------------------------------------------------
BincStream::~BincStream(void)
{
}

//------------------------------------------------------------------------
const string BincStream::str(void)
{
#ifndef HAVE_SSTREAM
  if (!terminated) {
    nstr << '\0';
    terminated = true;
  }
#endif

  return nstr.str();
}

//------------------------------------------------------------------------
void BincStream::clear(void)
{
#ifdef HAVE_SSTREAM
  nstr.str("");
#else
  nstr.clear();
  nstr.seekg(0);
  nstr.seekp(0);
  nstr.rdbuf()->freeze(0);
  terminated = false;
#endif  
}

//------------------------------------------------------------------------
int BincStream::getSize(void) const
{
#ifdef HAVE_SSTREAM
  return nstr.str().size();
#else
  return (((unsigned int)-1)>>1) - 1;
#endif
}
